/*
THE COMPUTER CODE CONTAINED HEREIN IS THE SOLE PROPERTY OF PARALLAX
SOFTWARE CORPORATION ("PARALLAX").  PARALLAX, IN DISTRIBUTING THE CODE TO
END-USERS, AND SUBJECT TO ALL OF THE TERMS AND CONDITIONS HEREIN, GRANTS A
ROYALTY-FREE, PERPETUAL LICENSE TO SUCH END-USERS FOR USE BY SUCH END-USERS
IN USING, DISPLAYING,  AND CREATING DERIVATIVE WORKS THEREOF, SO LONG AS
SUCH USE, DISPLAY OR CREATION IS FOR NON-COMMERCIAL, ROYALTY OR REVENUE
FREE PURPOSES.  IN NO EVENT SHALL THE END-USER USE THE COMPUTER CODE
CONTAINED HEREIN FOR REVENUE-BEARING PURPOSES.  THE END-USER UNDERSTANDS
AND AGREES TO THE TERMS HEREIN AND ACCEPTS THE SAME BY USE OF THIS FILE.  
COPYRIGHT 1993-1998 PARALLAX SOFTWARE CORPORATION.  ALL RIGHTS RESERVED.
*/
/*
 * $Source: Smoke:miner:source:main::RCS:config.c $
 * $Revision: 1.8 $
 * $Author: allender $
 * $Date: 1995/10/27 10:52:20 $
 * 
 * contains routine(s) to read in the configuration file which contains
 * game configuration stuff like detail level, sound card, etc
 * 
 * $Log: config.c $
 * Revision 1.8  1995/10/27  10:52:20  allender
 * call digi_set_master_volume when prefs are read in to
 * set the master volume of the mac
 *
 * Revision 1.7  1995/10/24  17:08:39  allender
 * Config_master_volume added for saving sound manager volume
 * across games
 *
 * Revision 1.6  1995/10/20  00:49:31  allender
 * use default values when no prefs file
 *
 * Revision 1.5  1995/09/21  10:06:58  allender
 * set digi and midi volume appropriately
 *
 * Revision 1.4  1995/09/13  08:49:38  allender
 * prefs file stuff
 *
 * Revision 1.3  1995/09/05  08:47:37  allender
 * prefs file working
 *
 * Revision 1.2  1995/05/26  06:54:14  allender
 * removed midi and digi references from config file
 *
 * Revision 1.1  1995/05/16  15:23:45  allender
 * Initial revision
 *
 * Revision 2.2  1995/03/27  09:42:59  john
 * Added VR Settings in config file.
 * 
 * Revision 2.1  1995/03/16  11:20:40  john
 * Put in support for Crystal Lake soundcard.
 * 
 * Revision 2.0  1995/02/27  11:30:13  john
 * New version 2.0, which has no anonymous unions, builds with
 * Watcom 10.0, and doesn't require parsing BITMAPS.TBL.
 * 
 * Revision 1.14  1995/02/11  16:19:36  john
 * Added code to make the default mission be the one last played.
 * 
 * Revision 1.13  1995/01/18  13:23:24  matt
 * Made curtom detail level vars initialize properly at load
 * 
 * Revision 1.12  1995/01/04  22:15:36  matt
 * Fixed stupid bug using scanf() to read bytes
 * 
 * Revision 1.11  1995/01/04  13:14:21  matt
 * Made custom detail level settings save in config file
 * 
 * Revision 1.10  1994/12/12  21:35:09  john
 * *** empty log message ***
 * 
 * Revision 1.9  1994/12/12  21:31:51  john
 * Made volume work better by making sure volumes are valid
 * and set correctly at program startup.
 * 
 * Revision 1.8  1994/12/12  13:58:01  john
 * MAde -nomusic work.
 * Fixed GUS hang at exit by deinitializing digi before midi.
 * 
 * Revision 1.7  1994/12/08  10:01:33  john
 * Changed the way the player callsign stuff works.
 * 
 * Revision 1.6  1994/12/01  11:24:07  john
 * Made volume/gamma/joystick sliders all be the same length.  0-->8.
 * 
 * Revision 1.5  1994/11/29  02:01:07  john
 * Added code to look at -volume command line arg.
 * 
 * Revision 1.4  1994/11/14  20:14:11  john
 * Fixed some warnings.
 * 
 * Revision 1.3  1994/11/14  19:51:01  john
 * Added joystick cal values to descent.cfg.
 * 
 * Revision 1.2  1994/11/14  17:53:09  allender
 * read and write descent.cfg file
 * 
 * Revision 1.1  1994/11/14  16:28:08  allender
 * Initial revision
 * 
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h> /* dir stuff */
#include <sys/stat.h>
#include <fcntl.h>

#include "dtypes.h"
#include "game.h"
#include "digi.h"
#include "kconfig.h"
#include "palette.h"
#include "joy.h"
#include "args.h"
#include "player.h"
#include "mission.h"
#include "strutil.h"

#include "macsys.h"

//WHS: save/load preferences stuff has been changed to use the player's .cfg (for whs.plr this is whs.cfg )

#if 0
static char *config_vr_type_str = "VR_type";
static char *config_vr_tracking_str = "VR_tracking";
#endif

#define MAX_CTB_LEN	512

typedef struct preferences {
	ubyte	digi_volume;
	ubyte	midi_volume;
	ubyte	stereo_reverse;
	ubyte	detail_level;
	ubyte	oc;					// object complexity
	ubyte	od;					// object detail
	ubyte	wd;					// wall detail
	ubyte	wrd;				// wall render depth
	ubyte	da;					// debris amount
	ubyte	sc;					// sound channels
	ubyte	gamma_level;
	ubyte	pixel_double;
	int		joy_axis_min[4];
	int		joy_axis_max[4];
	int		joy_axis_center[4];
	char	lastplayer[CALLSIGN_LEN+1];
	char	lastmission[MISSION_NAME_LEN+1];
	char	ctb_config[MAX_CTB_LEN];
	int		ctb_tool;
	ubyte	master_volume;
} Preferences;

char config_last_player[CALLSIGN_LEN+1] = "";
char config_last_mission[MISSION_NAME_LEN+1] = "";
char config_last_ctb_cfg[MAX_CTB_LEN] = "";
int config_last_ctb_tool;
ubyte Config_master_volume = 4;

int Config_vr_type = 0;
int Config_vr_tracking = 0;

extern byte	Object_complexity, Object_detail, Wall_detail, Wall_render_depth, Debris_amount, SoundChannels;
extern void digi_set_master_volume( int volume );

void set_custom_detail_vars(void);

static ubyte have_prefs = 0;



#define SAVE_PREFS_AS_TEXT

#ifdef SAVE_PREFS_AS_TEXT
//Descent.ini:
static char *last_player_str = "LastPlayer";

//'callsign'.cfg:
static char *digi_volume_str = "DigiVolume";
static char *midi_volume_str = "MidiVolume";

static char *detail_level_str = "DetailLevel";
static char *gamma_level_str = "GammaLevel";
static char *pixel_doubling_str = "PixelDoubling";

static char *stereo_rev_str = "StereoReverse";
static char *joystick_min_str = "JoystickMin";
static char *joystick_max_str = "JoystickMax";
static char *joystick_cen_str = "JoystickCen";
static char *last_mission_str = "LastMission";
static char *config_vr_type_str = "VR_type";
static char *config_vr_tracking_str = "VR_tracking";


//LastPlayer should be saved to descent.ini after a player change

int ReadConfigFile() { /* descent init, called in inferno.c */
	FILE *infile;
	char line[80], *token, *value, *ptr;

	if (!have_prefs) {	/* not yet initialized */
		strcpy( config_last_player, "" );
			/* get the last player to play and his preferences */
			/* or just set sound/midi loudness defaults? */
			/* also get any general statup stuff (like ?) */
		infile = fopen("Data/descent.cfg", "r"); //descent.ini ?
 	    while (!feof(infile)) {
		  memset(line, 0, 80);
		  fgets(line, 80, infile);
		  ptr = &(line[0]);
		  while (isspace(*ptr))
			ptr++;
		  if (*ptr != '\0') {
			token = strtok(ptr, "=");
			value = strtok(NULL, "=");
		    if (!strcmp(token, last_player_str))	{
			  strncpy( config_last_player, value, CALLSIGN_LEN );
			  strcutcrlf(config_last_player);
		    }
		  }
		}
		fclose(infile);
		have_prefs = 1;
		//Should now set the player callsign: Players[Player_num].callsign
		/* Now read the players config file */
		//		ReadPlayerConfigFile();
	}
	return 0;
}

int WriteConfigFile()
{
	FILE *infile;
	char str[256];

	infile = fopen("Data/descent.cfg", "w"); //descent.ini ?

	if (infile == NULL) {
		return 1;
	}
	sprintf (str, "%s=%s\n", last_player_str, Players[Player_num].callsign );
	fputs(str, infile);
	fclose(infile);
	return 0;
}

int ReadPlayerConfigFile()
{
	FILE *infile;
	char line[80], *token, *value, *ptr;
	ubyte gamma;
	int joy_axis_min[4];
	int joy_axis_center[4];
	int joy_axis_max[4];
	int i;

	char config_file[256];
	char player_cfg[CALLSIGN_LEN+5];

	strcpy(config_file, "Players/");

	strncpy(player_cfg, Players[Player_num].callsign, CALLSIGN_LEN);
	player_cfg[CALLSIGN_LEN]=0;
	strcat(player_cfg, ".cfg");
	strcat(config_file, player_cfg);

	joy_axis_min[0] = joy_axis_min[1] = joy_axis_min[2] = joy_axis_min[3] = 0;
	joy_axis_max[0] = joy_axis_max[1] = joy_axis_max[2] = joy_axis_max[3] = 0;
	joy_axis_center[0] = joy_axis_center[1] = joy_axis_center[2] = joy_axis_center[3] = 0;
	joy_set_cal_vals(joy_axis_min, joy_axis_center, joy_axis_max);

	Config_digi_volume = 4;
	Config_midi_volume = 4;
//	Config_control_type = 0; /* Not read in this file and thus should not be set! (gave a bug when going directly after startup to the options/controls menu) */
	Config_channels_reversed = 0;
//printf("ReadPlayerConfigFile: Config_control_type = 0\n");

//	infile = fopen("descent.cfg", "r");
	infile = fopen(config_file, "r");

	if (infile == NULL) {
		return 1;
	}
	while (!feof(infile)) {
		memset(line, 0, 80);
		fgets(line, 80, infile);
		ptr = &(line[0]);
		while (isspace(*ptr))
			ptr++;
		if (*ptr != '\0') {
			token = strtok(ptr, "=");
			value = strtok(NULL, "=");
			if (!strcmp(token, digi_volume_str))
				Config_digi_volume = strtol(value, NULL, 10);
			else if (!strcmp(token, midi_volume_str))
				Config_midi_volume = strtol(value, NULL, 10);
			else if (!strcmp(token, stereo_rev_str))
				Config_channels_reversed = strtol(value, NULL, 10);
			else if (!strcmp(token, gamma_level_str)) {
				gamma = strtol(value, NULL, 10);
				gr_palette_set_gamma( gamma );
			}
			else if (!strcmp(token, pixel_doubling_str)) {
				Scanline_double = strtol(value, NULL, 10);
			}
			else if (!strcmp(token, detail_level_str)) {
				Detail_level = strtol(value, NULL, 10);
				if (Detail_level == NUM_DETAIL_LEVELS-1) {
					int count,dummy,oc,od,wd,wrd,da,sc;

					count = sscanf (value, "%d,%d,%d,%d,%d,%d,%d\n",&dummy,&oc,&od,&wd,&wrd,&da,&sc);

					if (count == 7) {
						Object_complexity = oc;
						Object_detail = od;
						Wall_detail = wd;
						Wall_render_depth = wrd;
						Debris_amount = da;
						SoundChannels = sc;
						set_custom_detail_vars();
					}
				}
			}
			else if (!strcmp(token, joystick_min_str))	{
				sscanf( value, "%d,%d,%d,%d", &joy_axis_min[0], &joy_axis_min[1], &joy_axis_min[2], &joy_axis_min[3] );
			} 
			else if (!strcmp(token, joystick_max_str))	{
				sscanf( value, "%d,%d,%d,%d", &joy_axis_max[0], &joy_axis_max[1], &joy_axis_max[2], &joy_axis_max[3] );
			}
			else if (!strcmp(token, joystick_cen_str))	{
				sscanf( value, "%d,%d,%d,%d", &joy_axis_center[0], &joy_axis_center[1], &joy_axis_center[2], &joy_axis_center[3] );
			}
//			else if (!strcmp(token, last_player_str))	{
//				char * p;
//				strncpy( config_last_player, value, CALLSIGN_LEN );
//				p = strchr( config_last_player, '\n');
//				if ( p ) *p = 0;
//			}
			else if (!strcmp(token, last_mission_str))	{
				char * p;
				strncpy( config_last_mission, value, MISSION_NAME_LEN );
//				p = strchr( config_last_mission, '\n');
//				if ( p ) *p = 0;
				strcutcrlf(config_last_mission);

			} else if (!strcmp(token, config_vr_type_str)) {
				Config_vr_type = strtol(value, NULL, 10);
			} else if (!strcmp(token, config_vr_tracking_str)) {
				Config_vr_tracking = strtol(value, NULL, 10);
			}
		}
	}

	fclose(infile);

	i = FindArg( "-volume" );

	if ( i > 0 )	{
		i = atoi( Args[i+1] );
		if ( i < 0 ) i = 0;
		if ( i > 100 ) i = 100;
		Config_digi_volume = (i*8)/100;
		Config_midi_volume = (i*8)/100;
	}

	if ( Config_digi_volume > 8 ) Config_digi_volume = 8;

	if ( Config_midi_volume > 8 ) Config_midi_volume = 8;

	joy_set_cal_vals(joy_axis_min, joy_axis_center, joy_axis_max);
	digi_set_volume( (Config_digi_volume*32768)/8, (Config_midi_volume*128)/8 );

	return 0;
}

int WritePlayerConfigFile()
{
	FILE *infile;
	char str[256];
	int joy_axis_min[4];
	int joy_axis_center[4];
	int joy_axis_max[4];
	ubyte gamma = gr_palette_get_gamma();

	char config_file[256];
	char player_cfg[CALLSIGN_LEN+5];

	strcpy(config_file, "Players/");

	strncpy(player_cfg, Players[Player_num].callsign, CALLSIGN_LEN);
	player_cfg[CALLSIGN_LEN]=0;
	strcat(player_cfg, ".cfg");
	strcat(config_file, player_cfg);

		/* write */
	joy_get_cal_vals(joy_axis_min, joy_axis_center, joy_axis_max);

//	strncpy(config_file, Players[Player_num].callsign, CALLSIGN_LEN);
//	strcat(config_file, ".cfg");
	infile = fopen(config_file, "w");
//	infile = fopen("descent.cfg", "w");

	if (infile == NULL) {
		return 1;
	}
	sprintf (str, "%s=%d\n", digi_volume_str, Config_digi_volume);
	fputs(str, infile);
	sprintf (str, "%s=%d\n", midi_volume_str, Config_midi_volume);
	fputs(str, infile);
	sprintf (str, "%s=%d\n", stereo_rev_str, Config_channels_reversed);
	fputs(str, infile);
	sprintf (str, "%s=%d\n", gamma_level_str, gamma);
	fputs(str, infile);
	sprintf (str, "%s=%d\n", pixel_doubling_str, Scanline_double);
	fputs(str, infile);

	if (Detail_level == NUM_DETAIL_LEVELS-1)
		sprintf (str, "%s=%d,%d,%d,%d,%d,%d,%d\n", detail_level_str, Detail_level,
				Object_complexity,Object_detail,Wall_detail,Wall_render_depth,Debris_amount,SoundChannels);
	else
		sprintf (str, "%s=%d\n", detail_level_str, Detail_level);
	fputs(str, infile);
	sprintf (str, "%s=%d,%d,%d,%d\n", joystick_min_str, joy_axis_min[0], joy_axis_min[1], joy_axis_min[2], joy_axis_min[3] );
	fputs(str, infile);
	sprintf (str, "%s=%d,%d,%d,%d\n", joystick_cen_str, joy_axis_center[0], joy_axis_center[1], joy_axis_center[2], joy_axis_center[3] );
	fputs(str, infile);
	sprintf (str, "%s=%d,%d,%d,%d\n", joystick_max_str, joy_axis_max[0], joy_axis_max[1], joy_axis_max[2], joy_axis_max[3] );
	fputs(str, infile);
//	sprintf (str, "%s=%s\n", last_player_str, Players[Player_num].callsign );
//	fputs(str, infile);
	sprintf (str, "%s=%s\n", last_mission_str, config_last_mission );
	fputs(str, infile);
	sprintf (str, "%s=%d\n", config_vr_type_str, Config_vr_type );
	fputs(str, infile);
	sprintf (str, "%s=%d\n", config_vr_tracking_str, Config_vr_tracking );
	fputs(str, infile);
	fclose(infile);
	return 0;
}	


#else
/* Mac saves a struct, I prefer text files */
int ReadConfigFile()
{
	int i;
	int err;
	Preferences *prefs;
	char *p;
	int prefs_fp;

	prefs = (Preferences *) malloc((sizeof(Preferences)));

	err = read(prefs_fp, prefs, sizeof(Preferences));

	if (err<1) {
        //Set defaults

		return -1;
	}

	p = (char *)prefs;
	for (i = 0; i < sizeof(Preferences); i++) {
		if (*p != 0)
			break;
		p++;
	}
	if ( i == sizeof(Preferences) )
		return -1;

	Config_digi_volume = prefs->digi_volume;
	Config_midi_volume = prefs->midi_volume;
	Config_master_volume = prefs->master_volume;
	Config_channels_reversed = prefs->stereo_reverse;
	gr_palette_set_gamma( (int)(prefs->gamma_level) );
	Scanline_double = (int)prefs->pixel_double;
	Detail_level = prefs->detail_level;
	if (Detail_level == NUM_DETAIL_LEVELS-1) {
		Object_complexity = prefs->oc;
		Object_detail = prefs->od;
		Wall_detail = prefs->wd;
		Wall_render_depth = prefs->wrd;
		Debris_amount = prefs->da;
		SoundChannels = prefs->sc;
		set_custom_detail_vars();
	}
//	strncpy( config_last_player, prefs->lastplayer, CALLSIGN_LEN );
//	strcutcrlf(config_last_player);

//added to misc strutil.h:
// strcutcrlf(char *str) {
// p = strchr(config_last_player, 0x0d ); if (p) *p = 0;
// p = strchr(config_last_player, 0x0a ); if (p) *p = 0;
//}

	strncpy(config_last_mission, prefs->lastmission, MISSION_NAME_LEN);
	strcutcrlf(config_last_mission);

	strcpy(config_last_ctb_cfg, prefs->ctb_config);

	if ( Config_digi_volume > 8 ) Config_digi_volume = 8;

	if ( Config_midi_volume > 8 ) Config_midi_volume = 8;

	joy_set_cal_vals( prefs->joy_axis_min, prefs->joy_axis_center, prefs->joy_axis_max);
	digi_set_volume( (Config_digi_volume*256)/8, (Config_midi_volume*256)/8 );
	digi_set_master_volume(Config_master_volume);

	fclose(prefs_fp);
	return 0;
}


int WriteConfigFile()
{
	int err;
	Preferences *prefs;
    prefs = (Preferences *)(malloc(sizeof(Preferences)));

	joy_get_cal_vals(prefs->joy_axis_min, prefs->joy_axis_center, prefs->joy_axis_max);
	prefs->digi_volume = Config_digi_volume;
	prefs->midi_volume = Config_midi_volume;
	prefs->stereo_reverse = Config_channels_reversed;
	prefs->detail_level = Detail_level;
	if (Detail_level == NUM_DETAIL_LEVELS-1) {
		prefs->oc = Object_complexity;
		prefs->od = Object_detail;
		prefs->wd = Wall_detail;
		prefs->wrd = Wall_render_depth;
		prefs->da = Debris_amount;
		prefs->sc = SoundChannels;
	}
	prefs->gamma_level = (ubyte)gr_palette_get_gamma();
	prefs->pixel_double = (ubyte)Scanline_double;
	strncpy( prefs->lastplayer, Players[Player_num].callsign, CALLSIGN_LEN );
	strncpy( prefs->lastmission, config_last_mission, MISSION_NAME_LEN );
	strcpy( prefs->ctb_config, config_last_ctb_cfg);
	prefs->ctb_tool = config_last_ctb_tool;
	prefs->master_volume = Config_master_volume;

	err=fwrite(prefs, sizeof(Preferences), 1, prefs_fp);
	fclose(prefs_fp);

	return err;
}
#endif /* save as text/struct */






#if 0  /* WHS: not used, should use for binary/struct saving */
byte fread_byte(FILE *fp)
{
	byte b;

	fread(&b, sizeof(byte), 1, fp);
	return b;
}

short fread_short(FILE *fp)
{
	short s;
	
	fread(&s, sizeof(short), 1, fp);
	return swapshort(s);
}

int fread_int(FILE *fp)
{
	uint i;
	
	cfread(&i, sizeof(uint), 1, fp);
	return swapint(i);
}

fix fread_fix(FILE *fp)
{
	fix f;
	
	fread(&f, sizeof(fix), 1, fp);
	return (fix)swapint((uint)f);
}

#define fwrite_byte(a,b) write_byte(a,b)
//int write_byte(FILE *fp, byte b)
//{
//	return (fwrite(&b, sizeof(byte), 1, fp));
//}

#define fwrite_short(a,b) write_short_swap(a,b)
//int write_short_swap(FILE *fp, short s)
//{
//	s = swapshort(s);
//	return (fwrite(&s, sizeof(short), 1, fp));
//}

#define fwrite_int(a,b) write_int_swap(a,b)
//int write_int_swap(FILE *fp, int i)
//{
//	i = swapint(i);
//	return (fwrite(&i,sizeof(int), 1, fp));
//}

#define fwrite_fix(a,b) write_fix_swap(a,b)
//int write_fix_swap(FILE *fp, fix f)
//{
//	f = (fix)swapint((int)f);
//	return (fwrite(&f, sizeof(fix), 1, fp));
//}

#endif /* unused */
